//  Copyright (c) 1999-2008  Pavel Rychly

#include "concord.hh"
#include <cstdlib>
#include <cstring>

using namespace std;

//-------------------- simple_context --------------------
class simple_context : public Concordance::context {
public:
    class context_fun {
    public:
	virtual Position operator() (Position pos) = 0;
	virtual ~context_fun() {}
    };
protected:
    bool begin;
    int collnum;
    context_fun *fun;
public:
    simple_context (bool b, context_fun *f, int coll=0, int ch=0) 
	: context(ch), begin(b), collnum(coll), fun(f) {}
    virtual ~simple_context () {delete fun;};
    virtual Position get (Concordance *conc, ConcIndex idx);
};

Position simple_context::get (Concordance *conc, ConcIndex idx)
{
    return begin ? (*fun)(conc->coll_beg_at (collnum, idx)) :
	(*fun)(conc->coll_end_at (collnum, idx) -1);
}

//-------------------- ctx_* --------------------

class ctx_identity : public simple_context::context_fun {
public:
    ctx_identity () {}
    virtual Position operator() (Position pos) {return pos;}
};

class ctx_add_pos : public simple_context::context_fun {
    int delta;
public:
    ctx_add_pos (int delta) : delta (delta) {}
    virtual Position operator() (Position pos) {return pos + delta;}
};

class ctx_struct_beg : public simple_context::context_fun {
    ranges *rng;
    int delta;
public:
    ctx_struct_beg (ranges *rng, int delta) : rng (rng), delta (delta) {}
    virtual Position operator() (Position pos) {
        NumOfPos n = rng->num_at_pos (pos);
        if (n == -1)
            return -1;
        return rng->beg_at (min(max(NumOfPos(0), n + delta), rng->size()-1));
    }
};

class ctx_struct_end : public simple_context::context_fun {
    ranges *rng;
    int delta;
public:
    ctx_struct_end (ranges *rng, int delta) : rng (rng), delta (delta) {}
    virtual Position operator() (Position pos) {
        NumOfPos n = rng->num_at_pos (pos);
        if (n == -1)
            return -1;
        return rng->end_at (min(max(NumOfPos(0), n + delta), rng->size()-1)) -1;
    }
};


//-------------------- max,min_context --------------------

class max_context: public Concordance::context 
{
    Concordance::context *ctx1;
    Concordance::context *ctx2;
public:
    max_context (Concordance::context *c1, Concordance::context *c2)
	: Concordance::context(0), ctx1 (c1), ctx2 (c2) {}
    virtual ~max_context () {};
    virtual Position get (Concordance *conc, ConcIndex idx) {
	return max (ctx1->get (conc, idx), ctx2->get (conc, idx));
    }
};

class min_context: public Concordance::context 
{
    Concordance::context *ctx1;
    Concordance::context *ctx2;
public:
    min_context (Concordance::context *c1, Concordance::context *c2)
	: Concordance::context(0), ctx1 (c1), ctx2 (c2) {}
    virtual ~min_context () {delete ctx1; delete ctx2;}
    virtual Position get (Concordance *conc, ConcIndex idx) {
	return min (ctx1->get (conc, idx), ctx2->get (conc, idx));
    }
};


//-------------------- prepare_context --------------------
Concordance::context* 
Concordance::prepare_context (const char *ctxstr, bool toleft, int maxctx)
{
    if (!maxctx)
	maxctx = corp->get_maxctx();
    int num = atol (ctxstr);
    if (strchr (ctxstr, '#') != NULL) {
	if (num == 0)
	    return new simple_context (toleft, new ctx_add_pos (0), 0);
	if (num < 0)
	    num = - num;
	int npos = num/2 +1;
	if (maxctx && npos > maxctx) 
	    npos = maxctx;
 	return new simple_context (toleft, 
				   new ctx_add_pos (toleft ? - npos : npos),
				   0, num);
    } else {
	int coll = 0;
	bool fromdir = toleft;
	const char *p;
	if ((p = strchr (ctxstr, '<')) != NULL) {
	    fromdir = true;
	    if (*++p)
		coll = *p - '0';
	} else if ((p = strchr (ctxstr, '>')) != NULL) {
	    fromdir = false;
	    if (*++p)
		coll = *p - '0';
	}
	if ((p = strchr (ctxstr, ':')) != NULL) {
	    // structure name
	    char struct_name[80];
	    char *s = struct_name;
	    for (p++; isalpha (*p); p++, s++)
		*s = *p;
	    *s = '\0';
	    try {
		if (num == 0)
		    return new simple_context (fromdir, 
					       new ctx_add_pos (0), coll);
		bool numgt0 = num > 0;
		if (num < 0) num++;
		if (num > 0) num--;
		ranges *rng = corp->get_struct (struct_name)->rng;
		context *c;
		if (toleft)
		    c = new simple_context (fromdir, 
					    new ctx_struct_beg (rng, num), 
					    coll);
		else
		    c = new simple_context (fromdir, 
					    new ctx_struct_end (rng, num),
					    coll);
		if (!maxctx)
		    return c;
		else {
		    if (numgt0) {
			return new min_context (c, new simple_context
						(fromdir, 
						 new ctx_add_pos (maxctx),
						 coll));
		    } else {
			return new max_context (c, new simple_context
						(fromdir, 
						 new ctx_add_pos (-maxctx),
						 coll));
		    }

		}
		
	    } catch (AttrNotFound &e) {
		return new simple_context (toleft, 
					   new ctx_add_pos(toleft ? -3 : 3), 
					   coll);
	    }
	} else {
	    if (maxctx) {
		if (num > maxctx) {
		    num = maxctx;
		    if (toleft) num++;
		} else if (num <= - maxctx) {
		    num = - maxctx -1;
		    if (toleft) num++;
		}
	    }
	    return new simple_context (fromdir, new ctx_add_pos (num), coll);
	}
    }
}

