//  Copyright (c) 1999-2011  Pavel Rychly

#include "concord.hh"
#include <algorithm>
#include <sstream>
#include <cstdlib>
#include <cstring>

using namespace std;

typedef vector<pair<ConcIndex,ConcIndex> > VPCC;

void Concordance::delete_view_lines (const char *rngs)
{
    // vsechny data nechava, pouze meni view; (nepouziva se)
    // predpoklada vstup v indexech rng
    sync();
    ensure_view();
    Position f, t;
    istringstream in (rngs);
    VPCC rs;
    while (in >> f >> t)
        rs.push_back (pair<ConcIndex,ConcIndex>(f,t));
    stable_sort (rs.begin(), rs.end());
    vector<ConcIndex>::iterator beg = view->begin();
    for (VPCC::reverse_iterator i = rs.rbegin(); i != rs.rend(); i++)
        view->erase (beg + (*i).first, beg + (*i).second);
}

void Concordance::reduce_view_lines (const char *crit)
{
    // vsechny data nechava, pouze meni view; (nepouziva se)
    sync();
    ensure_view();
    double lines;
    char ch;
    istringstream in (crit);
    in >> lines;
    double scale = 1.0;
    while ((in >> ch) && ch == '%')
        scale /= 100;
    if (scale != 1.0)
        lines *= viewsize() * scale;

    vector<ConcIndex>::iterator beg = view->begin();
    float process = viewsize();
    for (Position i = viewsize()-1; i >= 0 && process > lines; process--, i--){
        if (process * rand() / (RAND_MAX + 1.0) > lines)
            view->erase (beg + i);
        else
            lines--;
    }
}

void Concordance::delete_lines (const char *rngs)
{
    sync();
    Position f, t;
    istringstream in (rngs);
    VPCC rs;
    while (in >> f >> t)
        if (f < t)
            rs.push_back (pair<Position,Position>(f,t));
    stable_sort (rs.begin(), rs.end());
    if (view) {
        vector<ConcIndex>::iterator beg = view->begin();
        for (VPCC::reverse_iterator i = rs.rbegin(); i != rs.rend(); i++) {
            for (Position v = (*i).first; v < (*i).second; v++) {
                ConcIndex idx = (*view)[v];
                rng[idx].beg = rng[idx].end = -1;
                for (unsigned int j=0; j < colls.size(); j++) {
                    if (colls[j][v].beg != cnotdef)
                        coll_count[j] --;
                }
            }
            view->erase (beg + (*i).first, beg + (*i).second);
        }
    } else {
        for (VPCC::reverse_iterator i = rs.rbegin(); i != rs.rend(); i++) {
            memmove (rng + (*i).first, rng + (*i).second,
                     (used - (*i).second) * sizeof (ConcItem));
            if (linegroup)
                linegroup->erase (linegroup->begin() + (*i).first,
                                  linegroup->begin() + (*i).second);
            if (!colls.empty()) {
                for (Position v = (*i).first; v < (*i).second; v++) {
                    for (unsigned int j=0; j < colls.size(); j++) {
                        if (colls[j][v].beg != cnotdef)
                            coll_count[j] --;
                    }
                }
                for (unsigned int j=0; j < colls.size(); j++) {
                    memmove (colls[j] + (*i).first, colls[j] + (*i).second,
                             (used - (*i).second) * sizeof (collocitem));
                }
            }
            used -=  (*i).second - (*i).first;
        }
    }
}

void Concordance::reduce_lines (const char *crit)
{
    sync();
    NumOfPos newsize;
    char ch;
    istringstream in (crit);
    in >> newsize;
    double scale = 1.0;
    while ((in >> ch) && ch == '%')
        scale /= 100;
    if (scale != 1.0)
        newsize = int (newsize * scale * viewsize());
    if (newsize >= viewsize())
        return;
    
    if (view) {
        delete view;
        view = NULL;
    }
    double lines = newsize;
    
    ConcItem *newrng = (ConcItem *) malloc (newsize * sizeof (ConcItem));
    //ConcItem *ri, *nri;
    vector<collocitem*> newcolls (colls.size());
    vector<ConcIndex> newcoll_count (colls.size());
    for (unsigned int i = 0; i < colls.size(); i++)
        if (colls[i])
            newcolls[i] = (collocitem*) malloc (newsize * sizeof (collocitem));

    float process = size();
    Position oldl, newl;
    for (oldl = newl = 0; oldl < size() && newl < newsize; oldl++, process--) {
        if (process * rand() / (RAND_MAX + 1.0) <= lines) {
            newrng [newl] = rng [oldl];
            for (unsigned int i = 0; i < colls.size(); i++) {
                if (colls[i]) {
                    newcolls[i][newl] = colls[i][oldl];
                    if (newcolls[i][newl].beg != cnotdef)
                        newcoll_count[i] ++;
                }
            }
            lines--;
            newl++;
        }
    }
    used = newsize;
    allocated = newsize;
    delete rng;
    rng = newrng;
    for (unsigned int i = 0; i < colls.size(); i++)
        free (colls[i]);
    colls = newcolls;
    coll_count = newcoll_count;
}

void Concordance::delete_pnfilter (int collnum, bool positive)
{
    ConcIndex newsize = coll_count [collnum -1];
    if (! positive)
        newsize = viewsize() - newsize;
    if (newsize == viewsize())
        return;

    vector<ConcIndex> *revview = NULL;
    if (view) {
        ConcIndex vs = view->size();
        revview = new vector<ConcIndex>(allocated, -1);
        for (ConcIndex i=0; i < vs; i++)
            (*revview)[(*view)[i]] = i;
        delete view;
        view = new vector<ConcIndex>(vs, -1);
    }
    ConcItem *newrng = (ConcItem *) malloc (newsize * sizeof (ConcItem));
    vector<collocitem*> newcolls (colls.size());
    vector<ConcIndex> newcoll_count (colls.size());
    for (unsigned int i = 0; i < colls.size(); i++)
        if (colls[i])
            newcolls[i] = (collocitem*) malloc (newsize * sizeof (collocitem));
    Position oldl, newl;
    for (oldl = newl = 0; oldl < size() && newl < newsize; oldl++) {
        if (rng[oldl].beg != -1 &&
            ((coll_beg_at (collnum, oldl) == -1) ^ positive)) {
            newrng [newl] = rng [oldl];
            for (unsigned int i = 0; i < colls.size(); i++) {
                if (colls[i]) {
                    newcolls[i][newl] = colls[i][oldl];
                    if (newcolls[i][newl].beg != cnotdef)
                        newcoll_count[i] ++;
                }
            }
            if (revview)
                (*view)[(*revview)[oldl]] = newl;
            newl++;
        }
    }

    used = newsize;
    allocated = newsize;
    delete rng;
    rng = newrng;
    for (unsigned int i = 0; i < colls.size(); i++)
        free (colls[i]);
    colls = newcolls;
    coll_count = newcoll_count;

    if (revview) {
        delete revview;
        vector<ConcIndex>::iterator vi = find (view->begin(), view->end(), -1);
        for (vector<ConcIndex>::iterator si = vi; si != view->end(); si++)
            if (*si != -1)
                *(vi++) = *si;
        if (vi != view->end())
            view->erase (vi, view->end());
    }
}

void Concordance::swap_kwic_coll (int collnum)
{
    if (collnum < 1 || unsigned (collnum) > colls.size() 
        || coll_count [collnum -1] == 0)
        return;
    ConcItem *ri = rng;
    collocitem *ci = colls [collnum -1];
    for (ConcItem *last = rng + used; ri < last; ri++, ci++) {
        if (ci->beg == cnotdef) continue;
        int oldend = ri->end;
        ri->end = ri->beg + ci->end;
        ri->beg += ci->beg;
        ci->beg = - ci->beg;
        ci->end = oldend - ri->beg;
    }
}

/*
 * Extends kwic to the given collocation and removes the collocation
 */
void Concordance::extend_kwic_coll (int collnum)
{
    if (collnum < 1 || unsigned (collnum) > colls.size()
        || coll_count [collnum -1] == 0)
        return;
    ConcItem *ri = rng;
    collocitem *ci = colls [collnum -1];
    for (ConcItem *last = rng + used; ri < last; ri++, ci++) {
        if (ci->beg == cnotdef) continue;
        if (ci->beg > 0)
            ri->end = ri->beg + ci->end;
        if (ci->beg < 0)
            ri->beg += ci->beg;
        if (ri->beg + ci->end > ri->end)
            ri->end = ri->beg + ci->end;
    }
    free (colls [collnum - 1]);
    colls[collnum - 1] = NULL;
    coll_count [collnum - 1] = 0;
}

void Concordance::shuffle()
{
    sync();
    if (!view)
        view = new vector<ConcIndex>(size());
    for (ConcIndex i=0; i < size(); i++)
        (*view)[i] = i;
    random_shuffle(view->begin(), view->end());
}

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
