//  Copyright (c) 1999-2007  Pavel Rychly

#ifndef CONCGET_HH
#define CONCGET_HH

#include "concord.hh"


//--------------------  --------------------
typedef std::vector<PosAttr*> PAvec;
class OutStruct;
class OutRef;
typedef std::vector<OutRef*> ORvec;
typedef std::vector<OutStruct*> OSvec;
typedef std::vector<std::string> Tokens;

//-------------------- KWICLines --------------------
class KWICLines {
protected:
    Concordance *conc;
    Concordance::context *leftctx, *rightctx;
    PAvec kwicattrs, ctxattrs;
    OSvec structs;
    ORvec refs;
    bool useview;
    bool inutf8;

    Position kwbeg, kwend, ctxbeg, ctxend;
    Tokens outrefs, outleft, outkwic, outright;
    ConcIndex lineidx;
public:
    KWICLines (Concordance *conc, const char *left, const char *right, 
	       const char *kwica, const char *ctxa, 
	       const char *struca, const char *refa, int maxctx=0, 
	       bool useview=true, bool ignore_nondef=true);
    ~KWICLines();
    bool nextcontext (ConcIndex linenum); // setting {kw,ctx}{beg,end} only
    bool nextline (ConcIndex linenum);  // nexcontext + setting out*
    const Position get_pos() {return kwbeg;}
    const int get_kwiclen() {return kwend - kwbeg;}
    const Position get_ctxbeg() {return ctxbeg;}
    const Position get_ctxend() {return ctxend;}
    const Tokens &get_ref_list() {return outrefs;}
    std::string get_refs();
    const Tokens &get_left() {return outleft;}
    const Tokens &get_kwic() {return outkwic;}
    const Tokens &get_right() {return outright;}
    const int get_linegroup();
};

//-------------------- CorpRegion --------------------
class CorpRegion {
protected:
    Corpus *corp;
    PAvec attrs;
    OSvec structs;
    bool ignore_nondef;
    Tokens output;
public:
    CorpRegion (Corpus *corp, const char *attra, const char *struca,
		bool ignore_nondef=true);
    ~CorpRegion();
    const Tokens &region (Position frompos, Position topos);
};

#endif
