//  Copyright (c) 1999-2011  Pavel Rychly

#include "concord.hh"
#include "cqpeval.hh"
#include "frsop.hh"
#include <cstdlib>
#include <cstring>

#ifndef HAVE_ZTHREAD
namespace ZThread {
    class Thread {
    public:
        Thread() {}
        virtual ~Thread() {}
        virtual void run() throw () =0;
        bool isCanceled() {return false;}
        void start() {run();}
        void join() {}
        void cancel() {}
    };
}
#endif

using namespace std;
using namespace ZThread;

void Concordance::free_rngmutex()
{
    Mutex *m = rng_mutex;
    rng_mutex = NULL;
    delete m;
    delete query;
    query = NULL;
}

void *evaluate_colloc (Concordance::set_collocation_data *data);


class EvalQueryThread: public Thread {
    Concordance *conc;
public:
    virtual void run() throw () {
        RangeStream *src = conc->query;
        conc->nestval = src->nesting();
        Position last = src->final();
        Position p;
        ConcIndex hardcut = conc->corp->get_hardcut();
        int lastcoll = conc->colls.size();
        if (!hardcut) 
            hardcut = -1; // no hardcut
        while ((p = src->peek_beg()) < last && hardcut--) {
            if (isCanceled()) {
                conc->free_rngmutex();
                return;
            }
            if (conc->used == conc->allocated) {
                size_t newsize = conc->used + 512;
                conc->lock();
                ConcItem *newrng = (ConcItem *) realloc (conc->rng, 
                                             newsize * sizeof (ConcItem));
                if (newrng) {
                    conc->rng = newrng;
                    conc->allocated = newsize;
                    for (int i=0; i < lastcoll; i++) {
                        Concordance::collocitem *cis = (Concordance::collocitem *)
                            realloc (conc->colls[i],
                                     newsize * sizeof (Concordance::collocitem));
                        if (cis)
                            conc->colls[i] = cis;
                        else {
                            // out of memory
                            conc->unlock();
                            conc->free_rngmutex();
                            return;
                        }
                    }
                    conc->unlock();
                } else {
                    // out of memory
                    conc->unlock();
                    conc->free_rngmutex();
                    return;
                }
            }
            conc->rng[conc->used].beg = p;
            conc->rng[conc->used].end = src->peek_end();
            if (lastcoll > 0) {
                Labels lab;
                src->add_labels(lab);
                for (int i=0; i < lastcoll; i++) {
                    Concordance::collocitem &ci = conc->colls[i][conc->used];
                    Labels::iterator labi = lab.find(i+1);
                    if (labi == lab.end())
                        ci.beg = ci.end = Concordance::cnotdef;
                    else {
                        ci.beg = labi->second - p;
                        ci.end = ci.beg + 1;
                        conc->coll_count[i]++;
                    }
                }
            }
            conc->used++;
            src->next();
        }
        conc->free_rngmutex();
        return;
    }
    EvalQueryThread (Concordance *c)
        : Thread (), conc (c)
    {}
    virtual ~EvalQueryThread () throw () {}
};

Concordance::Concordance (Corpus *corp, RangeStream *query, int inccolln)
    : rng (NULL), allocated (0), used (0), 
      view (NULL), linegroup (NULL), query (query), corp (corp), 
      corp_size (corp->size())
{
    if (!query)
        return;
    for (int i=0; i < inccolln; i++) {
        colls.push_back(NULL);
        coll_count.push_back(0);
    }
    rng_mutex = new Mutex();
    qeval = new EvalQueryThread (this);
    qeval->start();
}

Concordance::Concordance (Concordance &x)
    : view (NULL), linegroup (NULL), qeval (NULL), rng_mutex (NULL), 
      query (NULL), corp (x.corp), corp_size (x.corp_size)
{
    x.sync();
    nestval = x.nestval;
    allocated = x.allocated;
    used = x.used;

    rng = (ConcItem *) malloc (x.size() * sizeof (ConcItem));
    if (!rng)
        throw bad_alloc();
    memcpy (rng, x.rng, x.size() * sizeof (ConcItem));
    if (x.view)
        view = new vector<ConcIndex> (x.view->begin(), x.view->end());
    if (x.linegroup)
        linegroup = new linegroup_t (x.linegroup->begin(),
                                     x.linegroup->end());
    if (!x.colls.empty()) {
        for (unsigned int i=0; i < x.colls.size(); i++) {
            coll_count.push_back (x.coll_count[i]);
            colls.push_back ((collocitem *)
                             malloc (x.size() * sizeof (collocitem)));
            if (!colls[i])
                throw bad_alloc();
            memcpy (colls[i], x.colls[i], x.size() * sizeof (collocitem));
        }
    }
}

Concordance::~Concordance ()
{
    if (qeval) {
        qeval->cancel();
        qeval->join();
        delete qeval;
    }
    if (rng_mutex) {
        delete rng_mutex;
    }
    if (view)
        delete view;
    if (linegroup)
        delete linegroup;
    for (unsigned int i=0; i < colls.size(); i++)
        free (colls[i]);
    free (rng);
}

void Concordance::sync()
{
    if (qeval) {
        qeval->join();
        delete qeval;
        qeval = NULL;
    }
}

void Concordance::ensure_view ()
{
    if (!view) {
        view = new vector<ConcIndex>(size());
        for (ConcIndex i=0; i < size(); i++)
            (*view)[i] = i;
    }
}


//XXX void Concordance::ensure_compact ()
//        vyhazet -1

struct Concordance::set_collocation_data {
    string query;
    int collnum;
    Concordance *conc;
    context *leftctx, *rightctx;
    int rank;
    bool exclude_kwic;
    set_collocation_data (const string &q, int n, Concordance *c, int rn,
                          context *l, context *r, bool ek)
        : query (q), collnum (n), conc(c), leftctx (l), rightctx (r),
          rank (rn), exclude_kwic (ek)
    {}
};

void *evaluate_colloc (Concordance::set_collocation_data *data)
{
    Concordance *conc = data->conc;
    RangeStream *src = eval_cqpquery (data->query.c_str(), conc->corp);
    src = new RSFindBack (src);
    Concordance::collocitem *cis = (Concordance::collocitem *)
        malloc (sizeof (Concordance::collocitem) * conc->size());
    //XXX conc->nestval = src->nesting();
    int coll_count = 0;
    for (ConcIndex idx=0; idx < conc->size(); idx++) {
        if (conc->beg_at (idx) == -1) {
            cis[idx].beg = cis[idx].end = Concordance::cnotdef;
            continue;
        }
        Position ctxbeg = data->leftctx->get (conc, idx);
        Position ctxend = data->rightctx->get (conc, idx) +1;
        src->find_beg (ctxbeg);
        int r = data->rank;
        if (r == 0)
            r++;
        if (r > 0) {
            // r-ty
            Position b,e;
            while ((b = src->peek_beg()) >= ctxbeg && 
                   (e = src->peek_end()) <= ctxend && r) {
                if (!data->exclude_kwic
                    || (e <= conc->beg_at (idx) || b >= conc->end_at (idx)))
                    r--;
                if (r)
                    src->next();
            }
            e = src->peek_end();
            if (b >= ctxbeg && e <= ctxend && !r) {
                cis[idx].beg = b - conc->beg_at (idx);
                cis[idx].end = e - conc->beg_at (idx);
                coll_count++;
            } else {
                cis[idx].beg = cis[idx].end = Concordance::cnotdef;
            }
        } else {
            // posledni
            r = -r;
            vector <pair<Position, Position> > buff;
            buff.reserve (r);
            Position b,e;
            while ((b = src->peek_beg()) >= ctxbeg && 
                   (e = src->peek_end()) <= ctxend && int (buff.size()) < r) {
                if (!data->exclude_kwic
                    || (e <= conc->beg_at (idx) || b >= conc->end_at (idx)))
                    buff.push_back (make_pair(b,e));
                src->next();
            }
            while ((b = src->peek_beg()) >= ctxbeg && 
                   (e = src->peek_end()) <= ctxend) {
                if (!data->exclude_kwic
                    || (e <= conc->beg_at (idx) || b >= conc->end_at (idx))) {
                    buff.erase (buff.begin());
                    buff.push_back (make_pair(b,e));
                }
                src->next();
            }
            
            if (int (buff.size()) == r) {
                cis[idx].beg = buff[0].first - conc->beg_at (idx);
                cis[idx].end = buff[0].second - conc->beg_at (idx);
                coll_count++;
            } else {
                cis[idx].beg = cis[idx].end = Concordance::cnotdef;
            }
        }
    }

    conc->colls [data->collnum] = cis;
    conc->coll_count [data->collnum] = coll_count;
    delete src;
    delete data;
    return NULL;
}

void Concordance::set_collocation (int collnum, const string &cquery, 
                                   const char *lctx, const char *rctx, 
                                   int rank, bool exclude_kwic)
{
    sync();
    while (colls.size() < unsigned (collnum)) {
        colls.push_back (NULL);
        coll_count.push_back (0);
    }
    if (colls [--collnum]) {
        free (colls [collnum]);
        colls [collnum] = NULL;
        coll_count [collnum] = 0;
    }
    //qeval = new pthread_t;
    //int err = pthread_create (qeval, NULL, evaluate_colloc, 
    evaluate_colloc (
         new set_collocation_data (cquery, collnum, this, rank,
                                   prepare_context (lctx, true), 
                                   prepare_context (rctx, false), exclude_kwic));
}

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
