//  Copyright (c) 2003-2008  Pavel Rychly

#include <finlib/fromtof.hh>
#include <cstdlib>
#include "concord.hh"

using namespace std;

static const char concsignature[] = "\225ManateeConc\0\0\0";

enum {conctype_creating, conctype_ok};


Concordance::Concordance (Corpus *corp, const char *filename)
    : rng (NULL), allocated (0), used (0),
      view (NULL), linegroup (NULL), qeval (NULL), rng_mutex (NULL), 
      query (NULL), corp (corp), corp_size (corp->size())
{
    FILE *f = fopen (filename, "rb");
    if (!f)
	throw FileAccessError (filename, "Concordance::Concordance");
    {
	// check signature & type
	FromFile<char> ff(f);
	for (const char *c=concsignature; *c; c++) {
	    if (*c != ff.get())
		throw ConcNotFound (filename);
	}
	ff.seek (16);
	if (ff.get() != conctype_ok)
	    throw ConcNotFound (filename);
    }
    
    fseek (f, 32, SEEK_SET);
    fread (&allocated, sizeof (allocated), 1, f);
    rng = (ConcItem *) malloc (allocated * sizeof (ConcItem));
    used = fread (rng, sizeof (ConcItem), allocated, f);
    if (used != allocated)
	throw FileAccessError (filename, "Concordance::Concordance readx");
    
    // view
    ConcIndex csize;
    fread (&csize, sizeof (csize), 1, f);
    // FromFile can seek to a wrong place
    off_t after_view = ftell(f) + csize * sizeof(ConcIndex);
    if (csize && !feof(f)) {
	view = new vector<ConcIndex>();
	view->reserve (csize);
	FromFile<ConcIndex> ff (f);
	while (csize--)
	    view->push_back (ff.get());
    }
    fseek (f, after_view, SEEK_SET);

    // collocations
    fread (&csize, sizeof (csize), 1, f);
    while (csize && !feof(f)) {
	collocitem *ci = (collocitem *) malloc (csize * sizeof (collocitem));
	fread (ci, sizeof (collocitem), csize, f);
	coll_count.push_back (csize);
	colls.push_back (ci);
	fread (&csize, sizeof (csize), 1, f);
    }

    // linegroup
    fread (&csize, sizeof (csize), 1, f);
    if (csize) {
	linegroup = new linegroup_t();
	linegroup->reserve (csize);
	FromFile<short int> ff (f);
	while (csize--)
	    linegroup->push_back (ff.get());
    }

    fclose (f);
}


void Concordance::save (const char *filename, bool save_linegroup)
{
    sync();
    FILE *f = fopen (filename, "wb");
    fwrite (concsignature, sizeof (concsignature), 1, f);
    char conctype = conctype_ok;
    fwrite (&conctype, sizeof (conctype), 1, f);
    fseek (f, 32, SEEK_SET);

    fwrite (&used, sizeof (used), 1, f);
    fwrite (rng, sizeof (ConcItem), used, f);
    {
	// view
	ToFile<ConcIndex> ff (f);
	if (!view)
	    ff(0);
	else {
	    ff (view->size());
	    for (vector<ConcIndex>::iterator i=view->begin(); i != view->end(); i++)
		ff (*i);
	}
    }
    {
	// collocations
	ConcIndex colsize;
	vector<collocitem*>::iterator ci=colls.begin();
	for (vector<ConcIndex>::iterator cci=coll_count.begin();
	     cci != coll_count.end(); cci++, ci++) {
	    if (*cci && *ci) {
		colsize = *cci;
		fwrite (&colsize, sizeof (colsize), 1, f);
		fwrite (*ci, sizeof (collocitem), colsize, f);
	    }
	}
	colsize = 0;
	fwrite (&colsize, sizeof (colsize), 1, f);
    }
    if (save_linegroup) {
	// linegroup 
	{ // array size
	    ToFile<ConcIndex> ff (f);
	    if (!linegroup)
		ff(0);
	    else
		ff (linegroup->size());
	}
	if (linegroup) {
	    ToFile<short int> ff (f);
	    for (linegroup_t::iterator i=linegroup->begin(); 
		 i != linegroup->end(); i++)
		ff (*i);
	}
    } else {
	// empty linegroup
	ToFile<ConcIndex> ff (f);
	ff (0);
    }

    fclose (f);
}
