//  Copyright (c) 2007-2008  Pavel Rychly

#include "corpus.hh"
#include <sstream>
#include <iostream>

using namespace std;


class StructNums: public FastStream {
protected:
    ranges *rng;
    RangeStream *src;
    Position currnum;
    Position blockend;
    NumOfPos finval;
public:
    StructNums (ranges *rngi, RangeStream *srci)
	: rng(rngi), src(srci), currnum(-1), finval(rng->size()) {find(0);}
    virtual ~StructNums() {delete src;}
    virtual Position peek() {return currnum;}
    virtual Position next() {
	Position ret = currnum;
	if (currnum < finval && ++currnum > blockend)
	    find (currnum);
	return ret;
    }
    virtual Position find (Position pos) {
	if (currnum == finval || src->end()) {
	    currnum = finval;
	    blockend = finval;
	} else {
	    src->find_beg(rng->beg_at(pos));
	    Position s = rng->num_next_pos(src->peek_beg());
	    if (currnum < s)
		currnum = s;
	    blockend = rng->num_next_pos(src->peek_end());
	}
	return currnum;
    }
    virtual NumOfPos rest_min() {return src->rest_min();}
    virtual NumOfPos rest_max() {return src->rest_max();}
    virtual Position final() {return finval;}
};



RangeStream *Corpus::map_aligned (const string &al_corpname, Corpus *al_corp,
				  RangeStream *src)
{
    bool found = false;
    // check al_corpname in the aligned corpora list
    for (VSC::const_iterator i=aligned.begin(); i != aligned.end(); i++)
	if ((*i).first == al_corpname) {
	    al_corp = (*i).second;
	    found = true;
	    break;
	}

    if (!found) {
	// check al_corpname in the corpus config file
	istringstream as (get_conf("ALIGNED"));
	string cname; 
	while (getline (as, cname, ',')) {
	    if (cname.empty())
		continue;
	    if (al_corpname == cname) {
		if (!al_corp)
		    al_corp = new Corpus (al_corpname);
		aligned.push_back(pair<string,Corpus*>(al_corpname, al_corp));
		found = true;
		break;
	    }
	}
    }

    if (!found)
	throw CorpInfoNotFound (al_corpname + " not aligned");
    return get_struct("align")->rng->part
	(new StructNums(al_corp->get_struct("align")->rng, src));
}
